@@ -25,12 +25,13 @@ import json
 from calculate.core.server.gen_pid import search_worked_process2
 from calculate.core.setup_package import ChainProgressTemplate
 from calculate.lib.cl_template import templateFunction, SystemIni
-from calculate.lib.datavars import DataVars, Variable
+from calculate.lib.datavars import DataVars, Variable, DataVarsError
 from calculate.lib.utils.colortext import (TextState, get_color_print)
 from calculate.lib.utils.common import CmdlineParams, getTupleVersion
 from calculate.lib.utils.portage import (BinaryPackage, get_binary_file,
                                          EixVersionParser,
                                          clear_binhost_garbage, WorldFile)
+from calculate.lib.utils.binhosts import Binhosts
 from calculate.lib.utils.files import (
     pathJoin, PercentProgress, getProgPath, process, STDOUT, removeDir,
     makeDirectory, writeFile, readLinesFile, chmod, chown, FilePermission,
@@ -60,7 +61,7 @@ from calculate.update.emerge_parser import (EmergeParser,
 from calculate.lib.cl_log import log
 from calculate.update.update_tasks import EmergeMark
 from .build_storage import Build
-from calculate.update.update import Update
+from calculate.update.update import Update, variable_module
 from calculate.install.distr import (Distributive, IsoDistributive,
                                      DistributiveError, ContainerDistributive)
 import shutil
@@ -102,6 +103,11 @@ class Builder(Update):
         self.pkgnum = None
         self.pkgnummax = None
         self.world_data = ""
+        self.binhosts_data = None
+        self.refresh_binhost = False
+        self.gpgdata_md5 = []
+        self.gpg_changed = False
+        self.base = False
 
     def mount_target(self, target):
         dir_distro = target.convertToDirectory()
@@ -593,6 +599,10 @@ class Builder(Update):
         with self.clVars.useDefaultModule("update"):
             return super(Builder, self).regenCache(repname)
 
+    def prepare_gpg(self):
+        with self.clVars.useDefaultModule("update"):
+            return super(Builder, self).prepare_gpg()
+
     def getGit(self):
         chroot_path = self.clVars.Get('builder.cl_builder_path')
         sshkey = pathJoin(chroot_path,
@@ -624,6 +634,7 @@ class Builder(Update):
         makeDirectory(logpath)
         return True
 
+    @variable_module("builder")
     def _get_log_file(self):
         logname = "build-%s/%s" % (self.clVars.Get('cl_builder_id_path'),
                                    self.clVars.Get('cl_task_name'))
@@ -1107,12 +1118,13 @@ class Builder(Update):
             self._startEmerging(emerge)
         return True
 
-    def update_layman(self, builder_path):
+    def update_layman(self):
         """
         Обновить базу layman
         :param builder_path:
         :return:
         """
+        builder_path = self.clVars.Get("cl_builder_path")
         cmd = "/usr/bin/layman"
         cmd_path = self.get_prog_path(cmd)
         logfile = self._get_log_file()
@@ -1310,23 +1322,57 @@ class Builder(Update):
             raise BuilderError(str(e))
         return True
 
-    def raiseOutdate(self):
+    def prepare_update_vars(self):
         """
-        Установить флаг данные о репозиториях устарели (необходим для выполнения
-        eix-update и прочих команд обновляющих кэш
-        :return:
+        Заместить значения update переменных одноимёнными из builder
         """
-        self.clVars.Set('cl_builder_outdate_set', 'on', force=True)
+        vars_map = {
+            #Применить значение переменной для выбора веток репозиториев
+            #при обновлении
+            'update.cl_update_branch_name': 'builder.cl_builder_branch_name',
 
-    def apply_branch_variables(self):
-        """
-        Применить значение переменной для выбора веток репозиториев
-        при обновлении
-        """
-        self.clVars.Set('update.cl_update_branch_name',
-                        self.clVars.Get('builder.cl_builder_branch_name'))
+            'update.cl_update_gpg_force': 'builder.cl_builder_gpg_force',
+            'update.cl_update_gpg_keys': 'builder.cl_builder_gpg_keys',
+            'update.cl_update_binhost_revision_path': 
+                'builder.cl_builder_binhost_revision_path',
+            'update.cl_update_binhost_timestamp_path': 
+                'builder.cl_builder_binhost_timestamp_path',
+            'update.cl_update_last_timestamp': 
+                'builder.cl_builder_last_timestamp',
+
+            'update.cl_update_binhost_stable_set':
+                'builder.cl_builder_binhost_stable_set',
+            'update.cl_update_binhost_stable_opt_set':
+                'builder.cl_builder_binhost_stable_opt_set',
+
+            'update.cl_update_binhost':
+                'builder.cl_builder_binhost',
+
+            'update.cl_update_sync_rep': 'builder.cl_builder_sync_rep',
+            'update.cl_update_other_rep_name': 'builder.cl_builder_other_rep_name',
+            'update.cl_update_usetag_set': 'builder.cl_builder_usetag_set',
+
+            'update.cl_update_sync_overlay_rep': 'builder.cl_builder_sync_overlay_rep',
+            'update.cl_repository_name': 'builder.cl_builder_repository_name'
+        }
+        try:
+            for k,v in vars_map.items():
+                self.clVars.Set(k, self.clVars.Get(v), force=True)
+        except DataVarsError as e:
+            error = BuilderError(_("Failed to prepare variables for synchronization"))
+            error.addon = e
+            raise error
         return True
 
+    @variable_module("builder")
+    def _get_binhost_logger(self):
+        logname = "build-%s/%s" % (self.clVars.Get('cl_builder_id_path'),
+                                   "binhost-scan.log")
+        mainlog = self.clVars.Get('core.cl_log_path')
+        return log("binhost-scan.log",
+                   filename=path.join(mainlog, logname),
+                   formatter="%(message)s")
+
     def isohybrid(self, image_file):
         """
         Преобразовать ISO образ в гибридный
@@ -1390,6 +1436,9 @@ class Builder(Update):
             return False
         return True
 
+    def get_arch_machine(self):
+        return self.clVars.Get("builder.os_builder_arch_machine")
+
     @property
     def chroot_process(self):
         chroot_cmd = getProgPath("/usr/bin/chroot")
@@ -1510,11 +1559,12 @@ class Builder(Update):
         :return:
         """
         builder_path = self.clVars.Get('cl_builder_path')
-        programs = getRunCommands(chroot=builder_path)
+        programs = getRunCommands(chroot=builder_path, withpid=True)
         if programs:
+            pid, cmd = programs[0]
             raise BuilderError(
                 _("Chrooted {cmd} has already run into {id}").format(
-                    cmd=programs[0].split('\x00')[0],
+                    cmd="{}[{}]".format(cmd.split('\x00')[0], pid),
                     id=self.clVars.Get('cl_builder_id')
                 ))
         return True
@@ -1861,7 +1911,7 @@ class Builder(Update):
                 pretend = list(self.pretend_emerge_list(builder_path, package))
                 if not pretend:
                     self.endTask(False)
-                    self.printERROR(_("Failed to receive package list"))
+                    self.printERROR(_("Failed to receive the package list"))
                 else:
                     # получаем список пакетов которые необходимых для работы
                     required_pkgs = [x for x in required_pkgs
@@ -1907,20 +1957,20 @@ class Builder(Update):
         for pkg, reqs in automagic.items():
             if reqs:
                 self.printWARNING(
-                    _("Auto depends for {pkg} package by {pkgs} "
+                    _("Auto depends for package {pkg} from {pkgs} "
                       "will be used").format(
                         pkg=pkg, pkgs=",".join(str(x) for x in reqs)))
             clear_req_pkgs = automagic_clear.get(pkg, [])
             if clear_req_pkgs:
                 self.printWARNING(
-                    _("Auto depends for {pkg} package by {pkgs} "
-                      "will be clear").format(
+                    _("Auto depends for package {pkg} from {pkgs} "
+                      "will be cleared").format(
                         pkg=pkg, pkgs=",".join(str(x) for x in clear_req_pkgs)))
             waste_pkgs = automagic_waste.get(pkg, [])
             if waste_pkgs:
                 self.printWARNING(
-                    _("Specified depends for {pkg} package by {pkgs} "
-                      "obsoleted").format(
+                    _("Specified depends for package {pkg} from "
+                      "{pkgs} are obsolete").format(
                         pkg=pkg, pkgs=",".join(waste_pkgs)))
 
     def _rebuild_automagic(self, automagic, automagic_clear, builder_path):
@@ -2033,8 +2083,30 @@ class Builder(Update):
         """
         self.clVars.Write('cl_update_binhost', binhost)
         self.clVars.Set('cl_update_package_cache_set', 'on')
+        self.clVars.Set('cl_update_binhost_list', [binhost], force=True)
+        self.clVars.Set('cl_update_binhost_unstable_list', [binhost], force=True)
+        self.base = True
         return True
 
+    def create_binhost_data(self):
+        dv = self.clVars
+        last_ts = dv.Get('update.cl_update_last_timestamp')
+        if dv.GetBool('update.cl_update_binhost_stable_opt_set'):
+            binhost_list = dv.Get('update.cl_update_binhost_list')
+        else:
+            binhost_list = dv.Get('update.cl_update_binhost_unstable_list')
+        self.binhosts_data = Binhosts(
+            # значение малозначимо, поэтому берётся из собирающей системы
+            dv.GetInteger('update.cl_update_binhost_timeout'),
+            dv.Get('update.cl_update_binhost_revision_path'),
+            dv.Get('update.cl_update_binhost_timestamp_path'),
+            last_ts, binhost_list,
+            self.get_arch_machine(),
+            gpg=dv.Get('update.cl_update_gpg'),
+            base=self.base)
+        return True
+
+
     def index_pkgdir(self, pkgdir, trunkdir, stabledir):
         """
         Спрять pkgdir при использование base binhost, для корректного вычисления
@@ -2083,3 +2155,13 @@ class Builder(Update):
             raise BuilderError(
                 _("Failed to merge prepared binaries: %s") % str(e))
         return True
+
+    @variable_module("builder")
+    def update_binhost_list(self):
+        self.invalidateVariables('cl_builder_linux_datavars')
+        self.prepare_update_vars()
+        return super(Builder,self).update_binhost_list(
+                     self.clVars.Get('cl_builder_linux_datavars'))
+
+    def is_update_action(self, action):
+        return action == Actions.Update
