@@ -25,14 +25,17 @@ from calculate.lib.utils.dracut import Dracut
 from calculate.lib.utils.grub import GrubCommand
 from calculate.lib.utils.portage import getSquashList, isPkgInstalled
 from calculate.lib.variables.system import RootType
+import calculate.update.variables.update as update
 from .action import Actions
 import linux as vars_linux
 import datetime
 from calculate.install import distr
-from calculate.lib.utils.device import getUdevDeviceInfo, humanreadableSize
-from calculate.lib.utils.files import (isMount, process, typeFile,
+import calculate.lib.utils.device as device
+from calculate.lib.utils.device import humanreadableSize
+from calculate.lib.utils.files import (process, typeFile,
                                        listDirectory,
                                        pathJoin, readFile, check_rw)
+from calculate.lib.utils.mount import isMount
 from calculate.lib.utils.kernel import InitrdFile
 from calculate.lib.utils.text import _u8
 from calculate.lib.utils.tools import max_default
@@ -331,7 +334,8 @@ class VariableClBuilderDiskDev(Variable):
             raise VariableError(
                 _("You need to select a destination for build"))
         if value.startswith("/dev"):
-            cnDisk = getUdevDeviceInfo(name=value).get('DEVNAME', value)
+            cnDisk = device.udev.get_device_info(
+                name=value).get('DEVNAME', value)
             if cnDisk not in self.Get('install.os_disk_dev'):
                 raise VariableError(_("Wrong device '%s'") % value)
         if not value.startswith('/'):
@@ -1814,29 +1818,28 @@ class VariableClBuilderEixRepositories(ReadonlyVariable):
 
 
 class VariableClBuilderBinhostStableSet(
-    vars_linux.VariableClBuilderLinuxDatavars):
+    vars_linux.BuilderLinux, update.VariableClUpdateBinhostStableSet):
     """
     Использовать стабильное или нестабильное зеркало
     """
-    type = "bool"
-    value = "on"
-    mode = WRITEABLE
-
-    opt = ["--stable"]
+    variable = "cl_update_binhost_stable_set"
 
     def init(self):
         self.label = _("Use only stable updates")
-        self.help = _("use only stable updates")
 
-    def prepare_vars(self, dvbl):
-        pass
+    def get_fallback(self):
+        return Variable.On
+
+
+class VariableClBuilderBinhostStableOptSet(
+    update.VariableClUpdateBinhostStableOptSet):
+    """
+    Использовать стабильное или нестабильное зеркало
+    """
+    mode = Variable.Writable
 
     def get(self):
-        if self.Get('cl_action') in Actions.WorkAssemble:
-            dv = self.target_data()
-            if dv:
-                return dv.Get('update.cl_update_binhost_stable_opt_set')
-        return self.Get('update.cl_update_binhost_stable_opt_set')
+        return self.Get('cl_builder_binhost_stable_set')
 
 
 class VariableClBuilderBinhostScanSet(Variable):
